/* ##############################################################################################
**
** Copyright 2012-2016 CNRS, INPT
** Copyright 2013-2015 UPS
**  
** This file is part of qr_mumps.
**  
** qr_mumps is free software: you can redistribute it and/or modify
** it under the terms of the GNU Lesser General Public License as 
** published by the Free Software Foundation, either version 3 of 
** the License, or (at your option) any later version.
**  
** qr_mumps is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Lesser General Public License for more details.
**  
** You can find a copy of the GNU Lesser General Public License
** in the qr_mumps/doc directory.
**
** ##############################################################################################*/


/*##############################################################################################*/
/** @file qrm_mumps.h
 * Header file for the C interface
 *
 */
/*##############################################################################################*/

#if defined(zprec) || defined(cprec)
#include <complex.h>
#endif

#include <string.h>
#include "qrm_common_c.h"

struct sqrm_spmat_type_c{
  int              *irn, *jcn;
  float  *val;
  int               m, n, nz, sym;
  void             *h; 
};

struct sqrm_spfct_type_c{
  int            m;
  int            n;
  int            nz;
  int            sym;
  int           *cperm_in;
  int            icntl[20];
  float          rcntl[10];
  long long int  gstats[10];
  void          *h; 
};


int  sqrm_spmat_init_c(struct sqrm_spmat_type_c *qrm_spmat_c);
int  sqrm_spmat_destroy_c(struct sqrm_spmat_type_c *qrm_spmat_c);
int  sqrm_spfct_init_c(struct sqrm_spfct_type_c *qrm_spfct_c, struct sqrm_spmat_type_c *qrm_spmat_c);
int  sqrm_spfct_destroy_c(struct sqrm_spfct_type_c *qrm_spfct_c);
void sqrm_readmat_c(char *matfile, struct sqrm_spmat_type_c *qrm_spmat_c);
int  sqrm_analyse_c(struct sqrm_spmat_type_c *qrm_spmat_c,
                    struct sqrm_spfct_type_c *qrm_spfct_c,
                    const char transp);
int  sqrm_factorize_c(struct sqrm_spmat_type_c *qrm_spmat_c,
                      struct sqrm_spfct_type_c *qrm_spfct_c,
                      const char transp);
int  sqrm_solve_c(struct sqrm_spfct_type_c *qrm_spfct_c, const char transp,
                  float *b, float *x, const int nrhs);
int  sqrm_apply_c(struct sqrm_spfct_type_c *qrm_spfct_c, const char transp,
                  float *b, const int nrhs);
void sqrm_spmat_mv_c(struct sqrm_spmat_type_c *qrm_spmat_c, const char transp,
                     const float alpha, float *x, 
                     const float beta, float *y, 
                     const int nrhs);
int  sqrm_spmat_nrm_c(struct sqrm_spmat_type_c *qrm_spmat_c, const char ntype, 
                   float *nrm);
int  sqrm_vecnrm_c(const float *x, const int n, const int nrhs, 
                   const char ntype, float *nrm);
int  sqrm_spbackslash_c(struct sqrm_spmat_type_c *qrm_spmat_c, float *b, 
                          float *x, const int nrhs);
int  sqrm_spfct_backslash_c(struct sqrm_spfct_type_c *qrm_spfct_c, float *b, 
                          float *x, const int nrhs);
int  sqrm_spposv_c(struct sqrm_spmat_type_c *qrm_spmat_c, float *b, 
                          float *x, const int nrhs);
int  sqrm_least_squares_c(struct sqrm_spmat_type_c *qrm_spmat_c, float *b, 
                          float *x, const int nrhs);
int  sqrm_min_norm_c(struct sqrm_spmat_type_c *qrm_spmat_c, float *b, 
                          float *x, const int nrhs);
int  sqrm_residual_norm_c(struct sqrm_spmat_type_c *qrm_spmat_c, float *b, 
                          float *x, const int nrhs, float *nrm);
int  sqrm_residual_orth_c(struct sqrm_spmat_type_c *qrm_spmat_c, float *r, 
                          const int nrhs, float *nrm);


int  sqrm_spfct_set_i4_c (struct sqrm_spfct_type_c *qrm_spfct_c, const char *string, int val);
int  sqrm_spfct_set_r4_c (struct sqrm_spfct_type_c *qrm_spfct_c, const char *string, float val);
int  sqrm_spfct_get_i4_c (struct sqrm_spfct_type_c *qrm_spfct_c, const char *string, int *val);
int  sqrm_spfct_get_r4_c (struct sqrm_spfct_type_c *qrm_spfct_c, const char *string, float *val);
int  sqrm_spfct_get_i8_c(struct sqrm_spfct_type_c *qrm_spfct_c, const char *string, long long *val);
